/*

  irdecode.ino - software to assist in decoding an IR remote
  Written by Rylee Isitt
  Rev. 23 Feb 2014

  This softare is freeware, although I will not turn away donations ;)

*/

#include "nokiaLCD.h"

// ----------------------------------------------------------------------------------------------------
// The values of the variables below can be modified to configure the software for different hardware
// ----------------------------------------------------------------------------------------------------

// pinout configuration
// recommended pinout for the Arduino UNO
#define PIN_IR 2
#define PIN_LCD_SCE   8
#define PIN_LCD_RESET 9
#define PIN_LCD_DC    10
#define PIN_LCD_SDIN  11
#define PIN_LCD_SCLK  12
#define PIN_LCD_LED 13

// general remote configuration
#define REMOTE_RELEASEDELAY 100000L // elapsed time (in microseconds) after which if no bits have been received we assume that no button is being pressed
#define REMOTE_MINBITS 8 // the min number of bits to be received before a code can be considered received
#define REMOTE_MAXBITS 32 // the number of bits to be received upon which a code is considered received
#define REMOTE_HIGHMAX 2700UL // elapsed time (in microseconds) since last down-sloping IR signal edge above which the code reading will reset
#define REMOTE_HIGHMIN 1600UL // elapsed time (in microseconds) since last down-sloping IR signal edge that acts as a theshhold between a 0 and 1 bit
#define IR_TRIGGER FALLING // FALLING or RISING

// instantiate LCD
nokiaLCD lcd;

// variables for getting IR codes
// if using a remote that uses 16 bits or less for its code
// change the variable type of code and lastCode to unsigned int
volatile unsigned long lastCode = 0;
volatile unsigned long lastBitTimestamp = 0;
volatile byte lastBitCount = 0;

// ----------------------------------------------------------------------------------------------------
// This function is called by an interrupt when the IR receiver breakout board signal falls
// It measures the intervals between falling edges and uses these intervals to decode the IR signal
// ----------------------------------------------------------------------------------------------------

void irFire()
{
    /*
      Deciphering codes from the remote works by watching the elapsed time between down-sloping or up-sloping edges (as a high signal transitions to a low signal or vice versa)
      When this happens, irFire() is triggered by an interrupt
      ___     ____
         |___|    |____
         a        b
      In the waveform above, this elapsed time is the period between "a" and "b"
      If the elapsed time is between REMOTE_HIGHMIN and REMOTE_HIGHMAX, this is interpreted as a 1
      If the elapsed time is below REMOTE_HIGHMIN, this is interpreted as a 0
      If the elapsed time is above REMOTE_HIGHMAX, this resets code reading (sets code to 0 and bitCount to 0)
      Once REMOTE_BITS bits have been received, the code is stored in the lastCode variable
    */

    static unsigned long code = 0;
	static byte bitCount = 0;
    static unsigned long curTime = 0;
    curTime = micros();

    // watch the bits coming from the receiver
    if ((long)(curTime - lastBitTimestamp) < REMOTE_HIGHMIN)   // zero
    {
		bitCount++;
        code <<= 1;
    }
    else if ((long)(curTime - lastBitTimestamp) < REMOTE_HIGHMAX)     // one
    {
		bitCount++;
        code <<= 1;
        code++;
    }
    else     // begin a new code
    {
		if ((bitCount >= REMOTE_MINBITS) && (bitCount <= REMOTE_MAXBITS)) {
			lastCode = code;
			lastBitCount = bitCount;
		}
        code = 0;
		bitCount = 0;
    }

	// REMOTE_MAXBITS is reached, so capture code and begin anew
	if (bitCount == REMOTE_MAXBITS) {
		lastCode = code;
		lastBitCount = bitCount;
		bitCount = 0;
		code = 0;
	}
	
	lastBitTimestamp = curTime;

}

// ----------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------

void loop(void)
{

	// output the key code to the LCD
	if (lastCode != 0) {
        lcd.cls();
		lcd.writeLine_P(PSTR("Ready"));
        lcd.writeLine(String(lastCode));
		lcd.write(String(lastBitCount));
		lcd.writeLine_P(PSTR(" bits"));
		lastCode = 0;
	}

 
}

// ----------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------
// ----------------------------------------------------------------------------------------------------

void setup(void)
{

    // initialize LCD
    // BE SURE THAT THE FOLLOWING STATEMENT IS EXECUTED VERY EARLY ON IN THE PROGRAM
    // DAMAGE TO THE LCD CAN RESULT IF CARE IS NOT TAKEN TO DO SO WITHIN 100ms OF POWER-ON
    lcd.begin(PIN_LCD_SCE, PIN_LCD_RESET, PIN_LCD_DC, PIN_LCD_SDIN, PIN_LCD_SCLK, PIN_LCD_LED);
	lcd.cls();
	lcd.toggleBacklight();

    // setup pin for IR in
    pinMode(PIN_IR, INPUT);
    digitalWrite(PIN_IR, HIGH); // pulls high
    attachInterrupt(0, irFire, IR_TRIGGER);
    interrupts();
	
	// indicate ready status
	lcd.cls();
	lcd.writeLine_P(PSTR("Ready"));

}
